import re
import logging
import platform
import subprocess
from rich.logging import RichHandler

logging.basicConfig(
    level="NOTSET",
    format="%(message)s",
    datefmt="[%X]",
    handlers=[RichHandler(rich_tracebacks=False)],
)
log = logging.getLogger("rich")


class RequirementCheck:
    """
    Fidelius CLI requires JRE installed in the system (version 1.8+)
    ```py
    rcheck = RequirementCheck()
    res = rcheck.check()
    ```
    """

    def __init__(self) -> None:
        pass

    def __validate(self, outputs):
        version_line = outputs[0]
        match = re.search(r'version "?(\d+\.\d+)', version_line)
        if match:
            version = float(match.group(1))
            if version < 1.8:
                return False, "Java version is less than 1.8\nRequired: 1.8 or 1.8+"

        if not any(
            "Java(TM) SE Runtime Environment" in line
            or "OpenJDK Runtime Environment" in line
            for line in outputs
        ):
            return False, "Java Runtime Environment not found."

        if not any(
            "Java HotSpot(TM)" in line or "OpenJDK 64-Bit Server VM" in line
            for line in outputs
        ):
            return False, "Java VM not found."

        return True, "All requirements satisfied."

    def check(self):
        cli_command = subprocess.run(
            ["java", "-version"], stdout=subprocess.PIPE, stderr=subprocess.PIPE
        )
        response = cli_command.stderr  # Capture stderr instead of stdout
        outputs = response.decode("utf-8").split("\n")

        validation_res = self.__validate(outputs)

        if validation_res[0] is False:
            log.warning(validation_res[1])
            return False

        log.info(validation_res[1])
        return True

    def identify_os(self):
        os_name = platform.system()
        if os_name == "Linux":
            return "linux"
        elif os_name == "Windows":
            return "windows"
        elif os_name == "Darwin":
            return "mac"
        else:
            return "unknown"


# Example usage
rcheck = RequirementCheck()
os_identified = rcheck.identify_os()
log.info(f"Operating System identified: {os_identified}")
res = rcheck.check()



from pyfidelius.requirement_check import RequirementCheck


def test_check_jdk():
    req_check = RequirementCheck()
    assert req_check.check() == True


def test_check_os():
    req_check = RequirementCheck()
    assert req_check.identify_os() in ["linux", "windows", "mac"]



import os
from pyfidelius.cryptography import Cryptography

cryptography = Cryptography()


def test_generate_key_material():
    key_material = cryptography.generate_key_material()
    assert isinstance(key_material, dict)
    assert "privateKey" in key_material
    assert "x509PublicKey" in key_material
    assert "nonce" in key_material


def test_encrypt():
    data_to_encrypt = cryptography.encrypt(
        string_to_encrypt="Hello World!!",
        sender_nonce="lmXgblZwotx+DfBgKJF0lZXtAXgBEYr5khh79Zytr2Y=",
        requester_nonce="6uj1RdDUbcpI3lVMZvijkMC8Te20O4Bcyz0SyivX8Eg=",
        sender_private_key="AYhVZpbVeX4KS5Qm/W0+9Ye2q3rnVVGmqRICmseWni4=",
        requester_private_key="BAheD5rUqTy4V5xR4/6HWmYpopu5CO+KO8BECS0udNqUTSNo91TIqIIy1A4Vh+F94c+n9vAcwXU2bGcfsI5f69Y=",
    )
    assert isinstance(data_to_encrypt, dict)
    assert "encryptedData" in data_to_encrypt


def test_sane_encrypt():
    encrypted_data = cryptography.sane_encrypt(
        encoded_string_to_encrypt="SGVsbG8gV29ybGRweXRlc3QgLg==",
        sender_nonce="lmXgblZwotx+DfBgKJF0lZXtAXgBEYr5khh79Zytr2Y=",
        requester_nonce="6uj1RdDUbcpI3lVMZvijkMC8Te20O4Bcyz0SyivX8Eg=",
        sender_private_key="AYhVZpbVeX4KS5Qm/W0+9Ye2q3rnVVGmqRICmseWni4=",
        requester_private_key="BAheD5rUqTy4V5xR4/6HWmYpopu5CO+KO8BECS0udNqUTSNo91TIqIIy1A4Vh+F94c+n9vAcwXU2bGcfsI5f69Y=",
    )
    assert isinstance(encrypted_data, dict)
    assert "encryptedData" in encrypted_data


def test_decrypt():
    decryptes_data = cryptography.decrypt(
        encrypted_data="uDkxVIgMaNEht/VqkQI1cYFgS7xm+Gc6y8Qh8dU=",
        sender_nonce="6uj1RdDUbcpI3lVMZvijkMC8Te20O4Bcyz0SyivX8Eg=",
        requester_nonce="lmXgblZwotx+DfBgKJF0lZXtAXgBEYr5khh79Zytr2Y=",
        sender_private_key="DMxHPri8d7IT23KgLk281zZenMfVHSdeamq0RhwlIBk=",
        requester_public_key="BABVt+mpRLMXiQpIfEq6bj8hlXsdtXIxLsspmMgLNI1SR5mHgDVbjHO2A+U4QlMddGzqyEidzm1AkhtSxSO2Ahg=",
    )
    assert isinstance(decryptes_data, dict)
    assert "decryptedData" in decryptes_data
    assert decryptes_data["decryptedData"] == "Hello World!!"


def test_file_operation_encryption():
    file_path = os.path.join(os.path.dirname(__file__), "encode_test.txt")
    file_encrypted = cryptography.file_operation(file_path)
    assert isinstance(file_encrypted, dict)
    assert "encryptedData" in file_encrypted


def test_file_operation_decryption():
    file_path = os.path.join(os.path.dirname(__file__), "decode_test.txt")
    file_decrypted = cryptography.file_operation(file_path)
    assert isinstance(file_decrypted, dict)
    assert "decryptedData" in file_decrypted
    assert file_decrypted["decryptedData"] == "Hello World!!"



import subprocess
import os
import json
import base64
import logging
import platform
import re
from typing import List
from rich.logging import RichHandler

logging.basicConfig(
    level="NOTSET",
    format="%(message)s",
    datefmt="[%X]",
    handlers=[RichHandler(rich_tracebacks=False)],
)
log = logging.getLogger("rich")


class Cryptography:
    def __init__(self) -> None:
        current_file_path = os.path.dirname(os.path.abspath(__file__))
        root_path = os.path.abspath(os.path.join(current_file_path, ".."))

        self.os_platform = platform.system()

        if self.os_platform == "Windows":
            script_path = os.path.join(
                root_path, "fidelius-cli-1.2.0", "bin", "fidelius-cli.bat"
            )
        else:
            script_path = os.path.join(
                root_path, "fidelius-cli-1.2.0", "bin", "fidelius-cli"
            )

        self.SCRIPT_PATH = script_path

        if self.os_platform != "Windows":
            self.__prepare_script()

    def __prepare_script(self):
        """
        Prepares the script to be executable by fixing line endings and setting execute permissions.
        """
        try:
            # Fix line endings to Unix style (LF)
            with open(self.SCRIPT_PATH, "rb") as file:
                content = file.read()
            content = content.replace(b"\r\n", b"\n")  # Convert CRLF to LF

            with open(self.SCRIPT_PATH, "wb") as file:
                file.write(content)

            # Make the script executable
            subprocess.run(["chmod", "+x", self.SCRIPT_PATH], check=True)
        except Exception as e:
            log.error(f"Failed to prepare script. Error: {e}")
            raise

    def __execute_cli_commands(self, args: List[str]) -> str:
        command_to_execute = [self.SCRIPT_PATH] + args

        result = subprocess.run(
            command_to_execute,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            encoding="UTF-8",
        )

        if result.returncode != 0:
            raise Exception(f"Command failed with error: {result.stderr}")

        return result.stdout

    def __parse_output_to_dict(self, output: str) -> dict:
        pattern = r"\{.*?\}"
        match = re.search(pattern, output, re.DOTALL)

        if match:
            json_str = match.group(0)
            data_dict = json.loads(json_str)
            return data_dict
        else:
            raise ValueError("No valid JSON content found in the output")

    def __isBase64(self, value):
        try:
            if isinstance(value, str):
                sb_bytes = bytes(value, "ascii")
            elif isinstance(value, bytes):
                sb_bytes = value
            else:
                raise ValueError("Argument must be string or bytes")
            return base64.b64encode(base64.b64decode(sb_bytes)) == sb_bytes
        except Exception:
            return False

    def generate_key_material(self) -> dict:
        """
        Generates an ECDH key pair, and a random nonce.
        """
        try:
            output = self.__execute_cli_commands(["gkm"])
            key_materials = self.__parse_output_to_dict(output)
            return key_materials
        except Exception as e:
            log.error("Unable to generate key materials.\nError: ", e)
            return None

    def encrypt(
        self,
        string_to_encrypt,
        sender_nonce,
        requester_nonce,
        sender_private_key,
        requester_private_key,
    ):
        """
        Encrypts a given string data.

        Arguments:
        - `string_to_encrypt`: The string data to be encrypted.
        - `sender_nonce`: The nonce (number used once) of the sender for encryption.
        - `requester_nonce`: The nonce of the requester for encryption.
        - `sender_private_key`: The private key of the sender for encryption.
        - `requester_private_key`: The private key of the requester for encryption.

        Response:
        ```json
        {
            "encryptedData": "pzMvVZNNVtJzqPkkxcCbBUWgDEBy/mBXIeT2dJWI16ZAQnnXUb9lI+S4k8XK6mgZSKKSRIHkcNvJpllnBg548wUgavBa0vCRRwdL6kY6Yw=="
        }
        """
        try:
            if not all(
                map(
                    self.__isBase64,
                    [
                        sender_nonce,
                        requester_nonce,
                        sender_private_key,
                        requester_private_key,
                    ],
                )
            ):
                log.warning(
                    "sender_nonce, requester_nonce, sender_private_key, and requester_private_key must be base64 encoded"
                )
                return None
            output = self.__execute_cli_commands(
                [
                    "e",
                    string_to_encrypt,
                    sender_nonce,
                    requester_nonce,
                    sender_private_key,
                    requester_private_key,
                ]
            )

            encrypted_data = self.__parse_output_to_dict(output)
            return encrypted_data

        except Exception as e:
            log.error("Unable to encrypt message.\nError: ", e)
            return None

    def sane_encrypt(
        self,
        encoded_string_to_encrypt,
        sender_nonce,
        requester_nonce,
        sender_private_key,
        requester_private_key,
    ):
        """
        `sane-encrypt` is same as ncrypt command, with the only difference being that it accepts base64 encoded version of the input string.

        Arguments:
        - `encoded_string_to_encrypt`: The base64 encoded string that needs to be encrypted.
        - `sender_nonce`: The nonce (number used once) of the sender for encryption.
        - `requester_nonce`: The nonce of the requester for encryption.
        - `sender_private_key`: The private key of the sender for encryption.
        - `requester_private_key`: The private key of the requester for encryption.

        Response:
        ```json
        {
            "encryptedData": "pzMvVZNNVtJzqPkkxcCbBUWgDEBy/mBXIeT2dJWI16ZAQnnXUb9lI+S4k8XK6mgZSKKSRIHkcNvJpllnBg548wUgavBa0vCRRwdL6kY6Yw=="
        }
        """
        try:
            if not all(
                map(
                    self.__isBase64,
                    [
                        encoded_string_to_encrypt,
                        sender_nonce,
                        requester_nonce,
                        sender_private_key,
                        requester_private_key,
                    ],
                )
            ):
                log.warning(
                    "sender_nonce, requester_nonce, sender_private_key, and requester_private_key must be base64 encoded"
                )
                return None
            output = self.__execute_cli_commands(
                [
                    "e",
                    encoded_string_to_encrypt,
                    sender_nonce,
                    requester_nonce,
                    sender_private_key,
                    requester_private_key,
                ]
            )

            encrypted_data = self.__parse_output_to_dict(output)
            return encrypted_data

        except Exception as e:
            log.error("Unable to encrypt message.\nError: ", e)
            return None

    def decrypt(
        self,
        encrypted_data,
        sender_nonce,
        requester_nonce,
        sender_private_key,
        requester_public_key,
    ):
        """
        Decrypt the encoded data back to original string.

        Arguments:
        - `encrypted_data`: The encrypted data which is required to decrypt.
        - `sender_nonce`: The nonce (number used once) of the sender for encryption.
        - `requester_nonce`: The nonce of the requester for encryption.
        - `sender_private_key`: The private key of the sender for encryption.
        - `requester_public_key`: The public key of the requester for encryption.

        Response:
        ```json
        {
            "decryptedData": "Hello World"
        }
        """
        try:
            if not all(
                map(
                    self.__isBase64,
                    [
                        sender_nonce,
                        requester_nonce,
                        sender_private_key,
                        requester_public_key,
                    ],
                )
            ):
                log.warning(
                    "sender_nonce, requester_nonce, sender_private_key, and requester_public_key must be base64 encoded"
                )
                return None
            output = self.__execute_cli_commands(
                [
                    "d",
                    encrypted_data,
                    sender_nonce,
                    requester_nonce,
                    sender_private_key,
                    requester_public_key,
                ]
            )

            encrypted_data = self.__parse_output_to_dict(output)
            return encrypted_data

        except Exception as e:
            log.error("Unable to decrypt the encrypted data.\nError: ", e)
            return None

    def file_operation(self, file_path):
        """
        ECDH Cryptography can also be applied on file for both encryption and decryption.

        The file needs to be in certain formats

        - For encoding, text file should be in following format

        ```txt
        e
        <string-to-encrypt>
        <sender-nonce>
        <requester-nonce>
        <sender-private-key>
        <requester-public-key>
        ```

        - For decoding, text file should be in following format

        ```txt
        d
        <encrypted-data>
        <requester-nonce>
        <sender-nonce>
        <requester-private-key>
        <sender-public-key>
        ```
        """
        if not os.path.exists(file_path):
            log.error(f"File not found: {file_path}")
            return None

        with open(file_path, "r") as file:
            file_content = file.read()

        if file_content is None or file_content == "":
            log.error("File is empty.")
            return None

        file_content_list = file_content.split("\n")

        # Checks
        if len(file_content_list) > 6:
            log.error("No more than 6 lines are allowed")
            return None

        if file_content_list[0] not in ["e", "d"]:
            log.error(
                "Invalid operation. First line must be 'e' for encryption or 'd' for decryption"
            )
            return None

        if not all(
            map(
                self.__isBase64,
                file_content_list[2:],
            )
        ):
            log.warning(
                "sender_nonce, requester_nonce, sender_private_key, and requester_public_key must be base64 encoded"
            )
            return None

        output = self.__execute_cli_commands(file_content_list)

        processed_data = self.__parse_output_to_dict(output)

        return processed_data
